#include <windows.h>
#include "runtime.h"
#include "bit.h"

BITFILE* bitopen(BYTE* data, DWORD size)
{
	BITFILE* bf;

	bf = halloc(sizeof(BITFILE));

	bf->data = data;
	bf->byteofs = -1;
	bf->bitofs = 8;

	return bf;
}

int bitread(BITFILE* bf)
{
	int bit;

	if(bf->bitofs == 8)
	{
		if(bf->byteofs + 1 >= bf->size)
			return -1;
		bf->byteofs++;
		bf->bitofs = 0;
		bf->byte = bf->data[bf->byteofs];
	}

	bit = (bf->byte >> (7 - bf->bitofs)) & 0x01;
	bf->bitofs++;

	return bit;
}

int bitread8(BITFILE* bf)
{
	int byte = 0;
	int i;

	for(i = 0; i < 8; i++)
		byte = (byte << 1) | bitread(bf);

	return byte;
}

void bitclose(BITFILE* bf)
{
	hfree(bf);
}

#define N 256	// chars
int left[2*N-1], right[2*N-1];

int readtree(BITFILE* bf, BOOL init)
{
	int i;
	static int avail;

	if(init)
		avail = N;

	if(bitread(bf))	// 1=fushi
	{
		if((i = avail++) >= 2*N-1)
			MessageBox(NULL, "Incorrect tree", "Error", MB_ICONERROR | MB_OK);
		left[i] = readtree(bf, FALSE);
		right[i] = readtree(bf, FALSE);
		return i;
	}else
		return bitread8(bf);
}

void huffmanDecode(BYTE* dst, BYTE* src, DWORD dstsize, DWORD srcsize)
{
	BITFILE* bf;
	int root, j;
	DWORD k;

	bf = bitopen(src, srcsize);

	// read tree
	root = readtree(bf, TRUE);

	// decode
	for(k = 0; k < dstsize; k++)
	{
		j = root;
		while(j >= N)
		{
			if(bitread(bf))
				j = right[j];
			else
				j = left[j];
		}

		*dst++ = j;
	}

	bitclose(bf);
}
